import boto3, botocore, click
import os.path
import colorama as c

class Trapeze :

    def __init__(self, app_name, bucket) :
        c.init()
        self.s3 = boto3.client("s3")
        self.app_name = app_name
        self.bucket = bucket
        self.files = []

    def get_files(self) :
        files = self.s3.list_objects(Bucket = self.bucket, Prefix = self.app_name)
        return files

    def add_files(self, file_paths) :
        self.files = file_paths

    def push(self) :
        for f in self.files :
            try :
                key = "{0}/{1}".format(self.app_name, f)
                # exists?
                try :
                    exists = self.s3.head_object(Bucket = self.bucket, Key = key)
                except :
                    exists = None
                path = "{0}/{1}".format(self.bucket, key)
                if not exists or exists and click.confirm(c.Fore.RED + "'{0}' exists. Overwrite?".format(path) + c.Style.RESET_ALL) :
                    self.s3.upload_file(f, self.bucket, key)
                    click.echo("'{0}' written...".format(path))
            except FileNotFoundError :
                click.echo(c.Fore.RED + "File not found ({0}).".format(f) + c.Style.RESET_ALL)

    def pull(self) :
        for f in self.files :
            try :
                # file exists?
                exists = os.path.isfile(f)
                if not exists or exists and click.confirm(c.Fore.RED + "'{0}' exists. Overwrite?".format(f) + c.Style.RESET_ALL) :
                    key = "{0}/{1}".format(self.app_name, f)
                    self.s3.download_file(self.bucket, "{0}/{1}".format(self.app_name, f), f)
                    click.echo("'{0}' written...".format(f))
            except botocore.exceptions.ClientError :
                click.echo(c.Fore.RED + "Could not find remote file ({0}).".format(f) + c.Style.RESET_ALL)



import setuptools

with open("README.md", "r") as fh :
    long_description = fh.read()

setuptools.setup(
    name = "trapeze",
    version="0.0.24",
    author="Eric Gee",
    author_email="eric@gee.farm",
    description="Store/restore configuration and credentials for applications with AWS S3",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://bitbucket.org/ericgee/trapeze",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires = ['boto3','click'],
    entry_points = {
        'console_scripts': [
            'trapeze=trapeze.app:main'
        ]
    }
)



#!/usr/bin/env python
from .modules.trapeze import Trapeze
import json, argparse, click
from pathlib import Path
import colorama as c

@click.command()
@click.argument('action', type=click.Choice(['push','pull', 'init']))
@click.option('--config', default=".trapeze")
def main(action, config) :
    # initialize colorama for cli output
    c.init()

    if action == "init" :
        click.echo(c.Fore.BLACK + c.Back.GREEN + "Generating configuration file..." + c.Style.RESET_ALL)
        createConfig(config)
        exit

    # Open Config
    try :
        with open(config) as f :
            config_data = json.load(f)
    except FileNotFoundError :
        config_data = createConfig(config)

    # Init Trapeze
    t = Trapeze(config_data['name'], config_data['bucket'])
    files = config_data['files']
    t.add_files(files)

    # handle action
    click.echo(c.Fore.BLACK + c.Back.GREEN + "{0}ing to {1}...".format(action.capitalize(), config_data['bucket']) + c.Style.RESET_ALL)
    if action == "push" :
        t.push()
    elif action == "pull" :
        t.pull()

def createConfig(config) :
    if click.confirm("The config file ({0}) was not found.  Create one?".format(config), default = True) :
        # Open Global Config
        global_config_file = "{0}/.trapeze".format( str(Path.home()) )
        try :
            global_config_data = { "bucket": None }
            with open(global_config_file) as f :
                global_config_data = json.load(f)
        except FileNotFoundError :
            pass

        data = {
            "name": click.prompt("Application Name"),
            "bucket": click.prompt("AWS S3 Bucket", default = global_config_data['bucket'] ),
            "files": []
        }
        # get existing files
        t = Trapeze(data['name'], data['bucket'])
        objects = t.get_files()
        if "Contents" in objects :
            data['files'] = [ x['Key'].replace(data['name'] + "/", "") for x in objects['Contents'] ]
        click.echo(data)
        if click.confirm("Add files?", default=True) :
            # populate files
            click.echo("Which files would you like to track?")
            while True :
                filename = click.prompt("File Path (leave blank to abort)", default="")
                if filename :
                    data['files'].append(filename)
                    click.echo(data['files'])
                if not click.confirm("Add more?", default=True) :
                    break

        # save
        with open(config, 'w') as f :
            json.dump(data, f)

        return data
